@@ -215,11 +215,23 @@ bool RUFAnalysisVisitor::VisitFunctionDecl(FunctionDecl *FD)
       FD->getDependentSpecializationInfo();
     // don't need to track all specs, just associate FD with one
     // of those
-    if (Info->getNumTemplates() > 0) {
-      const FunctionDecl *Member =
-        Info->getTemplate(0)->getTemplatedDecl();
-      ConsumerInstance->addOneMemberSpecialization(FD, Member);
+    const FunctionDecl *Member = nullptr;
+
+#if LLVM_VERSION_MAJOR >= 18
+    ArrayRef<FunctionTemplateDecl *> members = Info->getCandidates();
+    if (!members.empty()) {
+      Member = members[0]->getTemplatedDecl();
     }
+#else
+    if (Info->getNumTemplates() > 0) {
+      Member =
+        Info->getTemplate(0)->getTemplatedDecl();
+    }
+#endif
+
+    if (Member != nullptr)
+      ConsumerInstance->addOneMemberSpecialization(FD, Member);
+
     return true;
   }
 
@@ -478,8 +490,24 @@ void RemoveUnusedFunction::removeOneFunctionDecl(const FunctionDecl *FD)
     LocEnd = SrcManager->getExpansionLoc(LocEnd);
   if (!FD->isInExternCContext() && !FD->isInExternCXXContext()) {
     SourceLocation FuncLocStart = getFunctionOuterLocStart(FD);
+
     if (FuncLocStart.isInvalid())
       return;
+    // This is a workaround to cases where we have function declaration
+    // groups, e.g.
+    //   void *foo(), bar();
+    // It's very tricky to remove a single decl in such a case.
+    // Let's just remove all decls for now. To avoid removing text that
+    // has been removed, we keep track the start loc for each group so that
+    // we only remove each group once.
+    if (!FD->hasBody()) {
+      if (FDGroupLocations.count(FuncLocStart)) {
+        return;
+      } else {
+        FDGroupLocations.insert(FuncLocStart);
+      }
+    }
+
     LocEnd = getFunctionLocEnd(FuncLocStart, LocEnd, FD);
     if (SrcManager->isWrittenInMainFile(FuncLocStart) &&
         SrcManager->isWrittenInMainFile(LocEnd))
@@ -636,7 +664,11 @@ void RemoveUnusedFunction::removeOneFunctionDeclGroup(const FunctionDecl *FD)
 {
   for (FunctionDecl::redecl_iterator RI = FD->redecls_begin(),
        RE = FD->redecls_end(); RI != RE; ++RI) {
-    removeOneFunctionDecl(*RI);
+    const FunctionDecl *FD = (*RI);
+    if (!FD->hasBody() && !FD->getTypeSourceInfo()) {
+      continue;
+    }
+    removeOneFunctionDecl(FD);
   }
 
   removeMemberSpecializations(FD);
@@ -649,7 +681,13 @@ void RemoveUnusedFunction::removeOneFunctionDeclGroup(const FunctionDecl *FD)
     const FunctionDecl *ParentFD = UsingParentFDs[(*I).first];
     if (ParentFD && RemovedFDs.count(ParentFD->getCanonicalDecl()))
       continue;
-    RewriteHelper->removeDecl((*I).first);
+    const UsingDecl *UD = (*I).first;
+    SourceRange Range = UD->getSourceRange();
+    if (Range.getBegin().isMacroID()) {
+      TheRewriter.RemoveText(SrcManager->getExpansionRange(Range));
+    } else {
+      RewriteHelper->removeDecl((*I).first);
+    }
   }
 }
 
@@ -888,11 +926,21 @@ void RemoveUnusedFunction::handleOneFunctionDecl(const FunctionDecl *TheFD)
       TheFD->getDependentSpecializationInfo();
     // don't need to track all specs, just associate FD with one
     // of those
-    if (Info->getNumTemplates() > 0) {
-      const FunctionDecl *Member =
-        Info->getTemplate(0)->getTemplatedDecl();
-      createFuncToExplicitSpecs(Member);
+    const FunctionDecl *Member = nullptr;
+
+#if LLVM_VERSION_MAJOR >= 18
+    ArrayRef<FunctionTemplateDecl *> members = Info->getCandidates();
+    if (!members.empty()) {
+      Member = members[0]->getTemplatedDecl();
     }
+#else
+    if (Info->getNumTemplates() > 0) {
+      Member = Info->getTemplate(0)->getTemplatedDecl();
+    }
+#endif
+
+    if (Member != nullptr)
+      createFuncToExplicitSpecs(Member);
     return;
   }
 
