@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -214,7 +214,11 @@ bool RewriteUtils::removeParamFromFuncDecl(const ParmVarDecl *PV,
   int RangeSize;
  
   SourceLocation StartLoc = ParamLocRange.getBegin();
+  if (StartLoc.isMacroID())
+    StartLoc = SrcManager->getExpansionLoc(StartLoc);
   SourceLocation EndLoc = ParamLocRange.getEnd();
+  if (EndLoc.isMacroID())
+    EndLoc = SrcManager->getExpansionLoc(EndLoc);
   if (StartLoc.isInvalid() && EndLoc.isInvalid()) {
     return false;
   }
@@ -232,7 +236,7 @@ bool RewriteUtils::removeParamFromFuncDecl(const ParmVarDecl *PV,
     }
   }
   else {
-    RangeSize = TheRewriter->getRangeSize(ParamLocRange);
+    RangeSize = TheRewriter->getRangeSize(SourceRange(StartLoc, EndLoc));
     if (RangeSize == -1)
       return false;
   }
@@ -538,7 +542,7 @@ bool RewriteUtils::removeVarFromDeclStmt(DeclStmt *DS,
 
   // VD is the the only declaration, so it is safe to remove the entire stmt
   if (DS->isSingleDecl()) {
-    return !(TheRewriter->RemoveText(StmtRange));
+    return !(TheRewriter->RemoveText(getRealSourceRange(StmtRange)));
   }
 
   // handle the case where we could have implicit declaration of RecordDecl
@@ -550,7 +554,7 @@ bool RewriteUtils::removeVarFromDeclStmt(DeclStmt *DS,
   // in this case, struct S0 is implicitly declared
   if (PrevDecl) {
     if ( RecordDecl *RD = dyn_cast<RecordDecl>(PrevDecl) ) {
-      DeclGroup DGroup = DS->getDeclGroup().getDeclGroup();
+      const DeclGroup &DGroup = DS->getDeclGroup().getDeclGroup();
       IsFirstDecl = true;
       if ((!RD->getDefinition() || RD->getNameAsString() == "") &&
           DGroup.size() == 2) {
@@ -676,8 +680,8 @@ bool RewriteUtils::replaceExpr(const Expr *E,
   int RangeSize = TheRewriter->getRangeSize(ExprRange);
   if (RangeSize == -1) {
     SourceLocation StartLoc = ExprRange.getBegin();
-    if (SrcManager->isMacroBodyExpansion(StartLoc)) {
-      StartLoc = SrcManager->getFileLoc(StartLoc);
+    if (StartLoc.isMacroID()) {
+      StartLoc = SrcManager->getExpansionLoc(StartLoc);
     }
     SourceLocation EndLoc = ExprRange.getEnd();
     if (SrcManager->isMacroBodyExpansion(EndLoc) ||
@@ -732,8 +736,8 @@ std::string RewriteUtils::getStmtIndentString(Stmt *S,
   StringRef MB = SrcManager->getBufferData(FID);
  
   unsigned lineNo = SrcManager->getLineNumber(FID, StartOffset) - 1;
-  const SrcMgr::ContentCache *
-      Content = SrcManager->getSLocEntry(FID).getFile().getContentCache();
+  const SrcMgr::ContentCache *Content =
+    &SrcManager->getSLocEntry(FID).getFile().getContentCache();
   unsigned lineOffs = Content->SourceLineCache[lineNo];
  
   // Find the whitespace at the start of the line.
@@ -744,7 +748,7 @@ std::string RewriteUtils::getStmtIndentString(Stmt *S,
     ++I;
   indentSpace = MB.substr(lineOffs, I-lineOffs);
 
-  return indentSpace;
+  return indentSpace.str();
 }
 
 bool RewriteUtils::addLocalVarToFunc(const std::string &VarStr,
@@ -969,6 +973,25 @@ bool RewriteUtils::replaceFunctionDeclName(const FunctionDecl *FD,
   // applying getNameAsString() on SomeClass() gives us SomeClass<T>.
 
   DeclarationNameInfo NameInfo = FD->getNameInfo();
+  if (const IdentifierInfo *IdInfo = FD->getLiteralIdentifier()) {
+    SourceLocation OpLoc = NameInfo.getBeginLoc();
+    const char *OpStartBuf = SrcManager->getCharacterData(OpLoc);
+
+    // skip string operator
+    std::string TmpStr(OpStartBuf,
+                       TheRewriter->getRangeSize(NameInfo.getSourceRange()));
+    std::string OperatorStr = "operator";
+    size_t Pos = TmpStr.find(OperatorStr);
+    TransAssert(Pos != std::string::npos && "cannot find operator!");
+
+    size_t OldNamePos =
+      TmpStr.find(IdInfo->getNameStart(), Pos, IdInfo->getLength());
+    TransAssert(OldNamePos != std::string::npos && "cannot find old name!");
+
+    SourceLocation NewStartLoc = OpLoc.getLocWithOffset(OldNamePos);
+    return !TheRewriter->ReplaceText(NewStartLoc, IdInfo->getLength(), NameStr);
+  }
+
   DeclarationName DeclName = NameInfo.getName();
   DeclarationName::NameKind K = DeclName.getNameKind();
   TransAssert((K != DeclarationName::CXXDestructorName) &&
@@ -1582,11 +1605,13 @@ SourceRange RewriteUtils::getFileLocSourceRange(SourceRange LocRange)
 {
   SourceLocation StartLoc = LocRange.getBegin();
   if (StartLoc.isMacroID()) {
-    StartLoc = SrcManager->getSpellingLoc(StartLoc);
-    SourceLocation EndLoc = LocRange.getEnd();
-    TransAssert(EndLoc.isMacroID() && "EndLoc is not from a macro!");
-    LocRange = SourceRange(StartLoc, SrcManager->getSpellingLoc(EndLoc));
+    StartLoc = SrcManager->getFileLoc(StartLoc);
   }
+  SourceLocation EndLoc = LocRange.getEnd();
+  if (EndLoc.isMacroID()) {
+    EndLoc = SrcManager->getFileLoc(EndLoc);
+  }
+  LocRange = SourceRange(StartLoc, EndLoc);
   return LocRange;
 }
 
