@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017, 2019 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -18,7 +18,9 @@
 
 #include "clang/AST/RecursiveASTVisitor.h"
 #include "clang/AST/ASTContext.h"
+#include "clang/AST/Mangle.h"
 #include "clang/Basic/SourceManager.h"
+#include "clang/Basic/TargetInfo.h"
 
 #include "TransformationManager.h"
 #include "CommonStatementVisitor.h"
@@ -214,10 +216,15 @@ bool SimpleInlinerFunctionStmtVisitor::VisitFunctionDecl(FunctionDecl *FD)
       !FD->isThisDeclarationADefinition())
     return true;
 
+  if (FD->getBeginLoc().isInvalid() || FD->getEndLoc().isInvalid())
+    return true;
   ConsumerInstance->CurrentFD = FD;
   ConsumerInstance->CollectionVisitor->setNumStmts(0);
   ConsumerInstance->CollectionVisitor->TraverseDecl(FD);
 
+  ConsumerInstance->
+    MangledNameToFuncDeclMap[ConsumerInstance->getMangledName(FD)] = FD;
+
   if (!FD->isVariadic()) {
     ConsumerInstance->FunctionDeclNumStmts[FD->getCanonicalDecl()] =
       ConsumerInstance->CollectionVisitor->getNumStmts();
@@ -245,6 +252,12 @@ void SimpleInliner::Initialize(ASTContext &context)
   FunctionVisitor = new SimpleInlinerFunctionVisitor(this);
   FunctionStmtVisitor = new SimpleInlinerFunctionStmtVisitor(this);
   StmtVisitor = new SimpleInlinerStmtVisitor(this);
+  if (context.getTargetInfo().getTriple().isOSWindows()) {
+    MangleCtx =
+      MicrosoftMangleContext::create(context, context.getDiagnostics());
+  } else {
+    MangleCtx = ItaniumMangleContext::create(context, context.getDiagnostics());
+  }
 }
 
 bool SimpleInliner::HandleTopLevelDecl(DeclGroupRef D)
@@ -274,7 +287,11 @@ void SimpleInliner::HandleTranslationUnit(ASTContext &Ctx)
   NameQueryWrap->TraverseDecl(Ctx.getTranslationUnitDecl());
   NamePostfix = NameQueryWrap->getMaxNamePostfix() + 1;
 
-  FunctionVisitor->TraverseDecl(CurrentFD);
+  if (FunctionDecl *AliaseeFD = getAliaseeFunctionDecl(CurrentFD)) {
+    FunctionVisitor->TraverseDecl(AliaseeFD);
+  } else {
+    FunctionVisitor->TraverseDecl(CurrentFD);
+  }
   StmtVisitor->TraverseDecl(TheCaller);
 
   TransAssert(TheStmt && "NULL TheStmt!");
@@ -285,6 +302,17 @@ void SimpleInliner::HandleTranslationUnit(ASTContext &Ctx)
     TransError = TransInternalError;
 }
 
+std::string SimpleInliner::getMangledName(FunctionDecl *FD) {
+  if (TransformationManager::isCLangOpt()) {
+    return FD->getNameAsString();
+  } else {
+    std::string S;
+    llvm::raw_string_ostream Stream(S);
+    MangleCtx->mangleName(FD, Stream);
+    return Stream.str();
+  }
+}
+
 bool SimpleInliner::isValidArgExpr(const Expr *E)
 {
   TransAssert(E && "NULL Expr!");
@@ -337,6 +365,12 @@ void SimpleInliner::getValidFunctionDecls(void)
   for (FunctionDeclToNumStmtsMap::iterator I = FunctionDeclNumStmts.begin(),
        E = FunctionDeclNumStmts.end(); I != E; ++I) {
     FunctionDecl *FD = (*I).first;
+    if (FD->hasDefiningAttr()) {
+      // skip bad alias
+      if (!getAliaseeFunctionDecl(FD)) {
+        continue;
+      }
+    }
     unsigned int NumStmts = (*I).second;
     unsigned int NumCalls = FunctionDeclNumCalls[FD];
 
@@ -359,6 +393,9 @@ void SimpleInliner::doAnalysis(void)
     FunctionDecl *CanonicalDecl = CalleeDecl->getCanonicalDecl();
     if (!ValidFunctionDecls.count(CanonicalDecl))
       continue;
+    // skip recursive call
+    if (CalleeDecl->getCanonicalDecl() == CalleeToCallerMap[(*CI)])
+      continue;
 
     if (!hasValidArgExprs(*CI))
       continue;
@@ -487,7 +524,7 @@ void SimpleInliner::sortReturnStmtsByOffs(const char *StartBuf,
   for (ReturnStmtsVector::iterator I = ReturnStmts.begin(),
        E = ReturnStmts.end(); I != E; ++I) {
     ReturnStmt *RS = (*I);
-    SourceLocation RSLocStart = RS->getLocStart();
+    SourceLocation RSLocStart = RS->getBeginLoc();
     const char *RSStartBuf = SrcManager->getCharacterData(RSLocStart);
     int Off = RSStartBuf - StartBuf;
     TransAssert((Off >= 0) && "Bad Offset!");
@@ -495,16 +532,36 @@ void SimpleInliner::sortReturnStmtsByOffs(const char *StartBuf,
   }
 }
 
+FunctionDecl *SimpleInliner::getAliaseeFunctionDecl(FunctionDecl *FD) {
+  if (const Attr *A = FD->getDefiningAttr()) {
+    const AliasAttr *Alias = cast<AliasAttr>(A);
+    llvm::StringRef Aliasee = Alias->getAliasee();
+    auto I = MangledNameToFuncDeclMap.find(Aliasee.str());
+    if (I == MangledNameToFuncDeclMap.end())
+      return nullptr;
+    else
+      return I->second;
+  } else {
+    return FD;
+  }
+}
+
 void SimpleInliner::copyFunctionBody(void)
 {
-  Stmt *Body = CurrentFD->getBody();
+  Stmt *Body;
+  FunctionDecl *AliaseeFD = getAliaseeFunctionDecl(CurrentFD);
+  if (AliaseeFD) {
+    Body = AliaseeFD->getBody();
+  } else {
+    Body = CurrentFD->getBody();
+  }
   TransAssert(Body && "NULL Body!");
 
   std::string FuncBodyStr("");
   RewriteHelper->getStmtString(Body, FuncBodyStr);
   TransAssert(FuncBodyStr[0] == '{');
 
-  SourceLocation StartLoc = Body->getLocStart();
+  SourceLocation StartLoc = Body->getBeginLoc();
   const char *StartBuf = SrcManager->getCharacterData(StartLoc);
 
   std::vector< std::pair<ReturnStmt *, int> > SortedReturnStmts;
@@ -571,8 +628,10 @@ void SimpleInliner::removeFunctionBody(void)
   }
 
   if (FunctionTemplateDecl *FTD = CurrentFD->getPrimaryTemplate()) {
-    TheRewriter.RemoveText(FTD->getSourceRange());
-    return;
+    if (FTD->getBeginLoc().isValid() && FTD->getEndLoc().isValid()) {
+      TheRewriter.RemoveText(FTD->getSourceRange());
+      return;
+    }
   }
 
   SourceRange FDRange = CurrentFD->getSourceRange();
