@@ -106,7 +106,7 @@ void Transformation::outputTransformedSource(llvm::raw_ostream &OutStream)
 void Transformation::outputOriginalSource(llvm::raw_ostream &OutStream)
 {
   FileID MainFileID = SrcManager->getMainFileID();
-  const llvm::MemoryBuffer *MainBuf = SrcManager->getBuffer(MainFileID);
+  auto MainBuf = SrcManager->getBufferOrNone(MainFileID);
   TransAssert(MainBuf && "Empty MainBuf!");
   OutStream << MainBuf->getBufferStart();
   OutStream.flush();
@@ -235,7 +235,9 @@ const Expr *Transformation::getInitExprFromBase(const Expr *BaseE,
 
   const ValueDecl *OrigDecl = DRE->getDecl();
   const VarDecl *VD = dyn_cast<VarDecl>(OrigDecl);
-  TransAssert(VD && "Bad VarDecl!");
+  if (!VD)
+    return NULL;
+
   const Type * Ty = VD->getType().getTypePtr();
   if (Ty->isPointerType())
     return NULL;
@@ -360,7 +362,7 @@ unsigned int Transformation::getConstArraySize(
   std::stringstream TmpSS(IntStr.str().str());
 
   if (!(TmpSS >> Sz)) {
-    TransAssert(0 && "Non-integer value!");
+    return UINT_MAX;
   }
   return Sz;
 }
@@ -395,8 +397,9 @@ const Expr *Transformation::getBaseExprAndIdxs(const Expr *E,
       // If we cannot have an integeral index, use 0.
       if (IdxE && IdxE->EvaluateAsInt(Result, *Context)) {
         llvm::APSInt IVal = Result.Val.getInt();
-        std::string IntStr = IVal.toString(10);
-        std::stringstream TmpSS(IntStr);
+        llvm::SmallString<8> IntStr;
+        IVal.toString(IntStr, 10);
+        std::stringstream TmpSS(IntStr.str().str());
         if (!(TmpSS >> Idx))
           TransAssert(0 && "Non-integer value!");
       }
@@ -804,7 +807,7 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
 
   case Type::TypeOf: {
     return getBaseDeclFromType(
-      dyn_cast<TypeOfType>(Ty)->getUnderlyingType().getTypePtr());
+      dyn_cast<TypeOfType>(Ty)->getUnmodifiedType().getTypePtr());
   }
 
   default:
@@ -873,8 +876,7 @@ bool Transformation::isBeforeColonColon(TypeLoc &Loc)
 }
 
 bool Transformation::replaceDependentNameString(const Type *Ty,
-                                                const TemplateArgument *Args,
-                                                unsigned NumArgs,
+                                                ArrayRef<TemplateArgument> Args,
                                                 std::string &Str,
                                                 bool &Typename)
 {
@@ -898,7 +900,7 @@ bool Transformation::replaceDependentNameString(const Type *Ty,
     return false;
 
   unsigned Idx = ParmTy->getIndex();
-  TransAssert((Idx < NumArgs) && "Bad Parm Index!");
+  TransAssert((Idx < Args.size()) && "Bad Parm Index!");
   const TemplateArgument Arg = Args[Idx];
   if (Arg.getKind() != TemplateArgument::Type)
     return false;
@@ -912,13 +914,12 @@ bool Transformation::replaceDependentNameString(const Type *Ty,
 
 bool Transformation::getTemplateTypeParmString(
        const TemplateTypeParmType *ParmTy,
-       const TemplateArgument *Args,
-       unsigned NumArgs,
+       ArrayRef<TemplateArgument> Args,
        std::string &Str)
 {
   unsigned Idx = ParmTy->getIndex();
   // we could have default template args, skip this case for now
-  if (Idx >= NumArgs)
+  if (Idx >= Args.size())
     return false;
   const TemplateArgument Arg = Args[Idx];
   if (Arg.getKind() != TemplateArgument::Type)
@@ -930,8 +931,7 @@ bool Transformation::getTemplateTypeParmString(
 
 bool Transformation::getTypedefString(const StringRef &Name,
                                       const CXXRecordDecl *CXXRD,
-                                      const TemplateArgument *Args,
-                                      unsigned NumArgs,
+                                      ArrayRef<TemplateArgument> Args,
                                       std::string &Str,
                                       bool &Typename)
 {
@@ -945,12 +945,12 @@ bool Transformation::getTypedefString(const StringRef &Name,
     Type::TypeClass TC = UnderlyingTy->getTypeClass();
     if (TC == Type::DependentName) {
       if (replaceDependentNameString(UnderlyingTy, Args,
-                                     NumArgs, Str, Typename))
+                                     Str, Typename))
         return true;
     }
     else if (const TemplateTypeParmType *ParmTy =
              UnderlyingTy->getAs<TemplateTypeParmType>()) {
-      if (getTemplateTypeParmString(ParmTy, Args, NumArgs, Str))
+      if (getTemplateTypeParmString(ParmTy, Args, Str))
         return true;
     }
   }
@@ -967,7 +967,7 @@ bool Transformation::getTypedefString(const StringRef &Name,
     const CXXRecordDecl *BaseDef = Base->getDefinition();
     if (!BaseDef)
       continue;
-    if (getTypedefString(Name, BaseDef, Args, NumArgs, Str, Typename))
+    if (getTypedefString(Name, BaseDef, Args, Str, Typename))
       return true;
   }
 
@@ -995,15 +995,13 @@ bool Transformation::getDependentNameTypeString(
   if (!BaseDef)
     return false;
 
-  unsigned NumArgs = 0;
-  const TemplateArgument *Args = NULL;
+  ArrayRef<TemplateArgument> Args;
   if (const TemplateSpecializationType *TST =
       Ty->getAs<TemplateSpecializationType>()) {
-    NumArgs = TST->getNumArgs();
-    Args = TST->getArgs();
+    Args = TST->template_arguments();
   }
   return getTypedefString(IdInfo->getName(),
-           BaseDef, Args, NumArgs, Str, Typename);
+           BaseDef, Args, Str, Typename);
 }
 
 bool Transformation::getTypeString(const QualType &QT,
